<?php
/*
Plugin Name: Cosmecos Plugin
Plugin URI: https://demo.artureanec.com/
Description: Register Custom Widgets and Custom Post Types for Cosmecos Theme.
Version: 1.1
Author: Artureanec
Author URI: https://demo.artureanec.com/
Text Domain: cosmecos_plugin
*/

// --- Register Custom Widgets --- //
if (!function_exists('cosmecos_widgets_load')) {
    function cosmecos_widgets_load() {
        require_once(__DIR__ . "/widgets/contacts.php");
        require_once(__DIR__ . "/widgets/featured-posts.php");
        require_once(__DIR__ . "/widgets/banner.php");
    }
}
add_action('plugins_loaded', 'cosmecos_widgets_load');

if (!function_exists('cosmecos_add_custom_widget')) {
    function cosmecos_add_custom_widget($name) {
        register_widget($name);
    }
}

// --- Register Custom Post Types --- //
if (!function_exists('cosmecos_register_custom_post_types')) {
    function cosmecos_register_custom_post_types() {
        # Portfolio
        register_taxonomy('portfolio-category', array('cosmecos-portfolio'), array(
            'hierarchical' => true,
            'label' => esc_html__('Portfolio Categories', 'cosmecos_plugin'),
            'singular_name' => esc_html__('Portfolio Category', 'cosmecos_plugin')
        ));
        register_post_type('cosmecos-portfolio', array(
                'label' => esc_html__('Portfolios', 'cosmecos_plugin'),
                'labels' => array(
                    'name' => esc_html__('Portfolios', 'cosmecos_plugin'),
                    'singular_name' => esc_html__('Portfolio', 'cosmecos_plugin'),
                    'all_items' => esc_html__('All Portfolios', 'cosmecos_plugin'),
                    'archives' => esc_html__('Portfolio', 'cosmecos_plugin')
                ),
                'public' => true,
                'show_ui' => true,
                'show_in_nav_menus' => true,
                'rewrite' => array(
                    'slug' => 'portfolio',
                    'with_front' => false
                ),
                'hierarchical' => true,
                'menu_position' => 4,
                'menu_icon' => 'dashicons-format-gallery',
                'supports' => array(
                    'title',
                    'editor',
                    'author',
                    'thumbnail',
                    'excerpt'
                ),
                'taxonomies' => array( 'portfolio-category' ),
                'has_archive' => true
            )
        );
    }
}

add_action('init', 'cosmecos_register_custom_post_types');

// Init Custom Widgets for Elementor
if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

final class Cosmecos_Custom_Widgets {
    const  VERSION = '1.1';
    const  MINIMUM_ELEMENTOR_VERSION = '2.0.0';
    const  MINIMUM_PHP_VERSION = '5.4';
    private static $_instance = null;

    public static function instance() {
        if (is_null(self::$_instance)) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }

    public function __construct() {
        add_action('plugins_loaded', [$this, 'init']);
    }

    public function i18n() {
        load_plugin_textdomain('cosmecos_plugin', false, plugin_basename(dirname(__FILE__)) . '/languages');
    }

    public function init() {
        // Check if Elementor installed and activated
        if (!did_action('elementor/loaded')) {
            add_action('admin_notices', [$this, 'cosmecos_admin_notice_missing_main_plugin']);
            return;
        }

        // Check for required Elementor version
        if (!version_compare(ELEMENTOR_VERSION, self::MINIMUM_ELEMENTOR_VERSION, '>=')) {
            add_action('admin_notices', [$this, 'cosmecos_admin_notice_minimum_elementor_version']);
            return;
        }

        // Check for required PHP version
        if (version_compare(PHP_VERSION, self::MINIMUM_PHP_VERSION, '<')) {
            add_action('admin_notices', [$this, 'cosmecos_admin_notice_minimum_php_version']);
            return;
        }

        // Add Custom Fonts Group
        if ( !function_exists('cosmecos_add_custom_fonts_group_to_elementor') ) {
            function cosmecos_add_custom_fonts_group_to_elementor($font_groups) {
                $additional_groups = array(
                    'local'     => __( 'Local', 'cosmecos' )
                );
                $font_groups = array_merge($font_groups, $additional_groups);
                return $font_groups;
            }
        }
        add_filter( 'elementor/fonts/groups', 'cosmecos_add_custom_fonts_group_to_elementor' );

        // Add Custom Fonts
        if ( !function_exists('cosmecos_add_custom_fonts_to_elementor') ) {
            function cosmecos_add_custom_fonts_to_elementor($fonts) {
                $additional_fonts = array(
                    'Quentin'   => 'local'
                );
                $fonts = array_merge($fonts, $additional_fonts);
                return $fonts;
            }
        }
        add_filter( 'elementor/fonts/additional_fonts', 'cosmecos_add_custom_fonts_to_elementor' );

        // Include Additional Files
        add_action('elementor/init', [$this, 'cosmecos_include_additional_files']);

        // Add new Elementor Categories
        add_action('elementor/init', [$this, 'cosmecos_add_elementor_category']);

        // Register Widget Scripts
        add_action('elementor/frontend/after_register_scripts', [$this, 'cosmecos_register_widget_scripts']);

        add_action('wp_enqueue_scripts', function () {
            wp_localize_script('ajax_query_products', 'cosmecos_ajaxurl',
                array(
                    'url' => admin_url('admin-ajax.php')
                )
            );
        });

        // Register New Widgets
        add_action('elementor/widgets/widgets_registered', [$this, 'cosmecos_widgets_register']);

        // Register Editor Styles
        add_action('elementor/editor/before_enqueue_scripts', function () {
            wp_register_style('cosmecos_elementor_admin', plugins_url('cosmecos-plugin/css/cosmecos-plugin-admin.css'));
            wp_enqueue_style('cosmecos_elementor_admin');
        });
    }


    public function cosmecos_admin_notice_missing_main_plugin() {
        $message = sprintf(
        /* translators: 1: Restbeef Core 2: Elementor */
            esc_html__('"%1$s" requires "%2$s" to be installed and activated.', 'cosmecos_plugin'),
            '<strong>' . esc_html__('Restbeef Core', 'cosmecos_plugin') . '</strong>',
            '<strong>' . esc_html__('Elementor', 'cosmecos_plugin') . '</strong>'
        );
        printf('<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message);
    }

    public function cosmecos_admin_notice_minimum_elementor_version() {
        $message = sprintf(
        /* translators: 1: Restbeef Core 2: Elementor 3: Required Elementor version */
            esc_html__('"%1$s" requires "%2$s" version %3$s or greater.', 'cosmecos_plugin'),
            '<strong>' . esc_html__('Restbeef Core', 'cosmecos_plugin') . '</strong>',
            '<strong>' . esc_html__('Elementor', 'cosmecos_plugin') . '</strong>',
            self::MINIMUM_ELEMENTOR_VERSION
        );
        printf('<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message);
    }

    public function cosmecos_admin_notice_minimum_php_version() {
        $message = sprintf(
        /* translators: 1: Press Elements 2: PHP 3: Required PHP version */
            esc_html__('"%1$s" requires "%2$s" version %3$s or greater.', 'cosmecos_plugin'),
            '<strong>' . esc_html__('Press Elements', 'cosmecos_plugin') . '</strong>',
            '<strong>' . esc_html__('PHP', 'cosmecos_plugin') . '</strong>',
            self::MINIMUM_PHP_VERSION
        );
        printf('<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message);
    }

    public function cosmecos_include_additional_files() {}

    public function cosmecos_add_elementor_category() {
        $categories = [];
        $categories['cosmecos_widgets'] =
            [
                'title' => esc_html__('Cosmecos Widgets', 'cosmecos_plugin'),
                'icon'  => 'fa fa-plug'
            ];
        $old_categories = \Elementor\Plugin::$instance->elements_manager->get_categories();
        $categories = array_merge($categories, $old_categories);

        $set_categories = function ( $categories ) {
            $this->categories = $categories;
        };
        $set_categories->call( \Elementor\Plugin::$instance->elements_manager, $categories );
    }

    public function cosmecos_register_widget_scripts() {
        // Lib
        wp_register_script('fancybox', plugins_url('cosmecos-plugin/js/lib/jquery.fancybox.min.js'), array('jquery'));
        wp_register_script('slick_slider', plugins_url('cosmecos-plugin/js/lib/slick.min.js'), array('jquery'));
        wp_register_script('isotope', plugins_url('cosmecos-plugin/js/lib/isotope.pkgd.min.js'), array('jquery', 'imagesloaded'));
        wp_register_script('plugin', plugins_url('cosmecos-plugin/js/lib/jquery.plugin.js'), array('jquery'));
        wp_register_script('countdown', plugins_url('cosmecos-plugin/js/lib/jquery.countdown.min.js'), array('jquery', 'plugin'));

        // Scripts
        wp_register_script('content_slider_widget', plugins_url('cosmecos-plugin/js/content-slider-widget.js'), array('jquery', 'slick_slider', 'fancybox'));
        wp_register_script('tabs_widget', plugins_url('cosmecos-plugin/js/tabs-widget.js'), array('jquery', 'fancybox'));
        wp_register_script('free_tabs_widget', plugins_url('cosmecos-plugin/js/free-tabs-widget.js'), array('jquery'));
        wp_register_script('countdown_widget', plugins_url('cosmecos-plugin/js/countdown-widget.js'), array('jquery', 'countdown'));
        wp_register_script('image_carousel_widget', plugins_url('cosmecos-plugin/js/image-carousel-widget.js'), array('jquery', 'swiper'));

        wp_register_script('elementor_widgets', plugins_url('cosmecos-plugin/js/elementor-widgets.js'), array('jquery', 'owl-carousel', 'isotope', 'cosmecos-theme'));
    }

    public function cosmecos_widgets_register() {

        // --- Include Widget Files --- //
        require_once __DIR__ . '/elements/banner.php';
        require_once __DIR__ . '/elements/blog.php';
        require_once __DIR__ . '/elements/button.php';
        require_once __DIR__ . '/elements/content-slider.php';
        require_once __DIR__ . '/elements/countdown.php';
        require_once __DIR__ . '/elements/heading.php';
        require_once __DIR__ . '/elements/icon-box.php';
        require_once __DIR__ . '/elements/image.php';
        require_once __DIR__ . '/elements/person.php';
        require_once __DIR__ . '/elements/portfolio-listing.php';
        require_once __DIR__ . '/elements/price-item.php';
        require_once __DIR__ . '/elements/promo.php';
        require_once __DIR__ . '/elements/special-text.php';
        require_once __DIR__ . '/elements/tabs.php';
        require_once __DIR__ . '/elements/testimonial-carousel.php';

        // --- Register Widgets --- //
        \Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Cosmecos\Widgets\Cosmecos_Banner_Widget());
        \Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Cosmecos\Widgets\Cosmecos_Blog_Listing_Widget());
        \Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Cosmecos\Widgets\Cosmecos_Button_Widget());
        \Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Cosmecos\Widgets\Cosmecos_Content_Slider_Widget());
        \Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Cosmecos\Widgets\Cosmecos_Countdown_Widget());
        \Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Cosmecos\Widgets\Cosmecos_Heading_Widget());
        \Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Cosmecos\Widgets\Cosmecos_Icon_Box_Widget());
        \Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Cosmecos\Widgets\Cosmecos_Image_Widget());
        \Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Cosmecos\Widgets\Cosmecos_Person_Widget());
        \Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Cosmecos\Widgets\Cosmecos_Portfolio_Listing_Widget());
        \Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Cosmecos\Widgets\Cosmecos_Price_Item_Widget());
        \Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Cosmecos\Widgets\Cosmecos_Promo_Widget());
        \Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Cosmecos\Widgets\Cosmecos_Special_Text_Widget());
        \Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Cosmecos\Widgets\Cosmecos_Tabs_Widget());
        \Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Cosmecos\Widgets\Cosmecos_Testimonial_Carousel_Widget());

        if (class_exists('WooCommerce')) {
            require_once __DIR__ . '/elements/products.php';
            require_once __DIR__ . '/elements/product-category.php';
            \Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Cosmecos\Widgets\Cosmecos_Products_Widget());
            \Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Cosmecos\Widgets\Cosmecos_Product_Category_Widget());
        }

        if (class_exists('WooCommerce') && shortcode_exists('pwb-az-listing')) {
            require_once __DIR__ . '/elements/brands-list.php';
            \Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Cosmecos\Widgets\Cosmecos_Brands_List_Widget());
        }

        if ( function_exists( 'wpforms' ) ) {
            require_once __DIR__ . '/elements/wpforms.php';
            \Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Cosmecos\Widgets\Cosmecos_Wpforms_Widget());
        }

        if ( function_exists( 'sb_instagram_feed_init' ) ) {
            require_once __DIR__ . '/elements/instagram-feed.php';
            \Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Cosmecos\Widgets\Cosmecos_Instagram_Feed_Widget());
        }
    }
}

Cosmecos_Custom_Widgets::instance();

if ( did_action('elementor/loaded') ) {
    add_action('elementor/widgets/widgets_registered', function ($widget_manager) {
        $widget_manager->unregister_widget_type('tabs');
    }, 15);
    add_action('elementor/element/before_section_end', function ($element, $section_id, $args) {
        if ('counter' === $element->get_name() && 'section_counter' === $section_id) {
            $element->add_responsive_control(
                'info_align',
                [
                    'label'     => esc_html__('Text Alignment', 'cosmecos_plugin'),
                    'type'      => \Elementor\Controls_Manager::CHOOSE,
                    'options'   => [
                        'left'      => [
                            'title'     => esc_html__('Left', 'cosmecos_plugin'),
                            'icon'      => 'fa fa-align-left',
                        ],
                        'center'    => [
                            'title'     => esc_html__('Center', 'cosmecos_plugin'),
                            'icon'      => 'fa fa-align-center',
                        ],
                        'right'     => [
                            'title'     => esc_html__('Right', 'cosmecos_plugin'),
                            'icon'      => 'fa fa-align-right',
                        ]
                    ],
                    'default'   => 'center',
                    'selectors' => [
                        '{{WRAPPER}} .elementor-counter .elementor-counter-title, {{WRAPPER}} .elementor-counter .elementor-counter-number-wrapper' => 'text-align: {{VALUE}};',
                    ],
                ]
            );
        }

        if ('image-carousel' === $element->get_name() && 'section_image_carousel' === $section_id) {
            $element->add_control(
                'view_style',
                [
                    'label'         => esc_html__('View Style', 'cosmecos_plugin'),
                    'type'          => \Elementor\Controls_Manager::SELECT,
                    'default'       => 'style_1',
                    'options'       => [
                        'style_1'       => esc_html__('Style 1', 'cosmecos_plugin'),
                        'style_2'       => esc_html__('Style 2', 'cosmecos_plugin')
                    ],
                    'separator'     => 'before',
                    'prefix_class'  => 'image_',
                    'toggle'        => false
                ]
            );
        }


        if ( 'video' === $element->get_name() && 'section_image_overlay' === $section_id ) {
            $element->add_control(
                'image_style',
                [
                    'label'         => esc_html__('Video Style', 'cosmecos_plugin'),
                    'type'          => \Elementor\Controls_Manager::SELECT,
                    'default'       => 'default',
                    'options'       => [
                        'default'               => esc_html__('Default', 'cosmecos_plugin'),
                        'framed'                => esc_html__('Framed', 'cosmecos_plugin')
                    ],
                    'prefix_class'  => 'video-image-style-',
                    'toggle'        => false,
                    'condition'     => [
                        'show_image_overlay'    => 'yes',
                        'image_overlay[url]!'   => '',
                    ],
                    'separator'     => 'before'
                ]
            );
            $element->add_control(
                'frame_color',
                [
                    'label'     => esc_html__('Frame Color', 'cosmecos_plugin'),
                    'type'      => \Elementor\Controls_Manager::COLOR,
                    'selectors' => [
                        '{{WRAPPER}}.video-image-style-framed .elementor-custom-embed-image-overlay:before' => 'border-color: {{VALUE}};'
                    ],
                    'condition' => [
                        'image_style' => 'framed'
                    ]
                ]
            );
        }
        if ( 'video' === $element->get_name() && 'section_video_style' === $section_id ) {
            $element->remove_control('play_icon_title');
            $element->remove_control('play_icon_color');
            $element->remove_responsive_control('play_icon_size');
            $element->remove_control('play_icon_text_shadow_color');
            $element->remove_control('play_icon_text_shadow_blur');
            $element->remove_control('play_icon_text_shadow_horizontal');
            $element->remove_control('play_icon_text_shadow_vertical');

            $element->add_responsive_control(
                'video_width',
                [
                    'label'         => esc_html__('Video Max Width', 'cosmecos_plugin'),
                    'type'          => \Elementor\Controls_Manager::SLIDER,
                    'size_units'    => [ 'px', '%' ],
                    'range'         => [
                        'px'            => [
                            'min'           => 100,
                            'max'           => 1170,
                        ],
                        '%'             => [
                            'min'           => 1,
                            'max'           => 100,
                        ]
                    ],
                    'selectors'     => [
                        '{{WRAPPER}} .elementor-widget-container' => 'max-width: {{SIZE}}{{UNIT}};'
                    ]
                ]
            );
            $element->add_responsive_control(
                'video_float',
                [
                    'label'     => esc_html__('Video Position', 'cosmecos_plugin'),
                    'type'      => \Elementor\Controls_Manager::CHOOSE,
                    'options'   => [
                        'left'      => [
                            'title'     => esc_html__('Left', 'cosmecos_plugin'),
                            'icon'      => 'fas fa-outdent',
                        ],
                        'none'      => [
                            'title'     => esc_html__('Center', 'cosmecos_plugin'),
                            'icon'      => 'fas fa-align-justify',
                        ],
                        'right'     => [
                            'title'     => esc_html__('Right', 'cosmecos_plugin'),
                            'icon'      => 'fas fa-indent',
                        ]
                    ],
                    'default'   => 'left',
                    'selectors' => [
                        '{{WRAPPER}} .elementor-widget-container' => 'float: {{VALUE}};',
                    ],
                ]
            );
            $element->add_control(
                'play_icon_title',
                [
                    'label' => esc_html__( 'Play Icon', 'elementor' ),
                    'type' => \Elementor\Controls_Manager::HEADING,
                    'condition' => [
                        'show_image_overlay' => 'yes',
                        'show_play_icon' => 'yes',
                    ],
                    'separator' => 'before',
                ]
            );

            $element->add_responsive_control(
                'play_button_size',
                [
                    'label' => esc_html__( 'Button Size', 'elementor' ),
                    'type' => \Elementor\Controls_Manager::SLIDER,
                    'range' => [
                        'px' => [
                            'min' => 10,
                            'max' => 300,
                        ],
                    ],
                    'selectors' => [
                        '{{WRAPPER}} .elementor-custom-embed-play i'        => 'width: {{SIZE}}{{UNIT}}; height: {{SIZE}}{{UNIT}};',
                        '{{WRAPPER}} .elementor-custom-embed-play i:before' => 'line-height: {{SIZE}}{{UNIT}};'
                    ],
                    'condition' => [
                        'show_image_overlay' => 'yes',
                        'show_play_icon' => 'yes',
                    ],
                ]
            );

            $element->add_responsive_control(
                'play_icon_size',
                [
                    'label' => esc_html__( 'Icon Size', 'elementor' ),
                    'type' => \Elementor\Controls_Manager::SLIDER,
                    'range' => [
                        'px' => [
                            'min' => 10,
                            'max' => 300,
                        ],
                    ],
                    'selectors' => [
                        '{{WRAPPER}} .elementor-custom-embed-play i:before' => 'font-size: {{SIZE}}{{UNIT}}',
                    ],
                    'condition' => [
                        'show_image_overlay' => 'yes',
                        'show_play_icon' => 'yes',
                    ],
                ]
            );

            $element->start_controls_tabs('button_settings_tabs');

                // ------------------------ //
                // ------ Normal Tab ------ //
                // ------------------------ //
                $element->start_controls_tab(
                    'tab_button_normal',
                    [
                        'label' => esc_html__('Normal', 'cosmecos_plugin')
                    ]
                );

                    $element->add_control(
                        'play_icon_color',
                        [
                            'label'     => esc_html__('Icon Color', 'cosmecos_plugin'),
                            'type'      => \Elementor\Controls_Manager::COLOR,
                            'selectors' => [
                                '{{WRAPPER}} .elementor-custom-embed-play i' => 'color: {{VALUE}};'
                            ]
                        ]
                    );

                    $element->add_control(
                        'play_icon_bg',
                        [
                            'label'     => esc_html__('Icon Background', 'cosmecos_plugin'),
                            'type'      => \Elementor\Controls_Manager::COLOR,
                            'selectors' => [
                                '{{WRAPPER}} .elementor-custom-embed-play i' => 'background: {{VALUE}};'
                            ]
                        ]
                    );

                $element->end_controls_tab();

                // ----------------------- //
                // ------ Hover Tab ------ //
                // ----------------------- //
                $element->start_controls_tab(
                    'tab_button_hover',
                    [
                        'label' => esc_html__('Hover', 'cosmecos_plugin')
                    ]
                );

                    $element->add_control(
                        'play_icon_hover',
                        [
                            'label'     => esc_html__('Icon Hover', 'cosmecos_plugin'),
                            'type'      => \Elementor\Controls_Manager::COLOR,
                            'selectors' => [
                                '{{WRAPPER}} .elementor-custom-embed-play:hover i' => 'color: {{VALUE}};'
                            ]
                        ]
                    );

                    $element->add_control(
                        'play_icon_bg_hover',
                        [
                            'label'     => esc_html__('Icon Background Hover', 'cosmecos_plugin'),
                            'type'      => \Elementor\Controls_Manager::COLOR,
                            'selectors' => [
                                '{{WRAPPER}} .elementor-custom-embed-play:hover i' => 'background: {{VALUE}};'
                            ]
                        ]
                    );

                $element->end_controls_tab();

            $element->end_controls_tabs();
        }


        if ('image-carousel' === $element->get_name() && 'section_style_navigation' === $section_id) {
            $element->remove_control('dots_position');
            $element->remove_control('dots_size');
            $element->remove_control('dots_color');
            $element->start_controls_tabs('button_settings_tabs');

            // ------------------------ //
            // ------ Normal Tab ------ //
            // ------------------------ //
            $element->start_controls_tab(
                'pagination_normal',
                [
                    'label' => esc_html__('Normal', 'cosmecos_plugin')
                ]
            );

            $element->add_control(
                'dot_color',
                [
                    'label'     => esc_html__('Pagination Dot Color', 'cosmecos_plugin'),
                    'type'      => \Elementor\Controls_Manager::COLOR,
                    'selectors' => [
                        '{{WRAPPER}} .swiper-pagination-bullet:after' => 'background: {{VALUE}};'
                    ]
                ]
            );

            $element->add_control(
                'dot_border',
                [
                    'label'     => esc_html__('Pagination Dot Border', 'cosmecos_plugin'),
                    'type'      => \Elementor\Controls_Manager::COLOR,
                    'selectors' => [
                        '{{WRAPPER}} .swiper-pagination-bullet' => 'border-color: {{VALUE}};'
                    ]
                ]
            );

            $element->end_controls_tab();

            // ------------------------ //
            // ------ Active Tab ------ //
            // ------------------------ //
            $element->start_controls_tab(
                'pagination_active',
                [
                    'label' => esc_html__('Active', 'cosmecos_plugin')
                ]
            );

            $element->add_control(
                'dot_active',
                [
                    'label'     => esc_html__('Pagination Active Dot Color', 'cosmecos_plugin'),
                    'type'      => \Elementor\Controls_Manager::COLOR,
                    'selectors' => [
                        '{{WRAPPER}} .swiper-pagination-bullet.swiper-pagination-bullet-active:after' => 'background: {{VALUE}};'
                    ]
                ]
            );

            $element->add_control(
                'dot_border_active',
                [
                    'label'     => esc_html__('Pagination Active Dot Border', 'cosmecos_plugin'),
                    'type'      => \Elementor\Controls_Manager::COLOR,
                    'selectors' => [
                        '{{WRAPPER}} .swiper-pagination-bullet.swiper-pagination-bullet-active' => 'border-color: {{VALUE}};'
                    ]
                ]
            );

            $element->end_controls_tab();

            $element->end_controls_tabs();
        }

        if ('image-carousel' === $element->get_name() && 'section_style_image' === $section_id) {
            $element->remove_control('image_spacing');
            $element->remove_control('image_spacing_custom');
            $element->add_group_control(
                \Elementor\Group_Control_Box_Shadow::get_type(),
                [
                    'name'      => 'image_shadow',
                    'selector'  => '{{WRAPPER}} .elementor-image-carousel-wrapper .elementor-image-carousel .swiper-slide-image',
                ]
            );

            $element->add_control(
                'image_spacing',
                [
                    'label'     => esc_html__( 'Spacing', 'cosmecos_plugin' ),
                    'type'      => \Elementor\Controls_Manager::SELECT,
                    'options'   => [
                        ''          => esc_html__( 'Default', 'cosmecos_plugin' ),
                        'custom'    => esc_html__( 'Custom', 'cosmecos_plugin' ),
                    ],
                    'default'   => '',
                    'condition' => [
                        'slides_to_show!' => '1',
                    ],
                ]
            );
            $element->add_control(
                'image_spacing_custom',
                [
                    'label'                 => esc_html__( 'Image Spacing', 'cosmecos_plugin' ),
                    'type'                  => \Elementor\Controls_Manager::SLIDER,
                    'range'                 => [
                        'px'                    => [
                            'max'                   => 100,
                        ],
                    ],
                    'default'               => [
                        'size'                  => 20,
                    ],
                    'show_label'            => false,
                    'condition'             => [
                        'image_spacing'         => 'custom',
                        'slides_to_show!'       => '1',
                    ],
                    'frontend_available'    => true,
                    'render_type'           => 'none',
                    'separator'             => 'after',
                    'selectors'             => [
                        '{{WRAPPER}} .elementor-image-carousel-wrapper' => 'margin: -{{SIZE}}px; padding: {{SIZE}}px !important;'
                    ],
                ]
            );

            $element->add_control(
                'icon_color',
                [
                    'label'     => esc_html__('Icon Color', 'cosmecos_plugin'),
                    'type'      => \Elementor\Controls_Manager::COLOR,
                    'selectors' => [
                        '{{WRAPPER}} .swiper-slide a:after' => 'color: {{VALUE}};'
                    ],
                    'separator' => 'before',
                    'condition' => [
                        'link_to'   => ['file', 'custom']
                    ]
                ]
            );

            $element->add_control(
                'icon_bg_color',
                [
                    'label'     => esc_html__('Icon Background Color', 'cosmecos_plugin'),
                    'type'      => \Elementor\Controls_Manager::COLOR,
                    'selectors' => [
                        '{{WRAPPER}} .swiper-slide a:after' => 'background-color: {{VALUE}};'
                    ],
                    'condition' => [
                        'link_to'   => ['file', 'custom']
                    ]
                ]
            );

            $element->add_control(
                'overlay_color',
                [
                    'label'     => esc_html__('Image Overlay Color', 'cosmecos_plugin'),
                    'type'      => \Elementor\Controls_Manager::COLOR,
                    'selectors' => [
                        '{{WRAPPER}} .swiper-slide a:before' => 'background-color: {{VALUE}};'
                    ],
                    'condition' => [
                        'link_to'       => ['file', 'custom'],
                        'view_style'    => 'style_2'
                    ]
                ]
            );


            $element->start_controls_tabs('frame_settings_tabs');

                // ------------------------ //
                // ------ Normal Tab ------ //
                // ------------------------ //
                $element->start_controls_tab(
                    'frame_normal',
                    [
                        'label'     => esc_html__('Normal', 'cosmecos_plugin'),
                        'condition' => [
                            'link_to'    => ['file', 'custom'],
                            'view_style' => 'style_2'
                        ]
                    ]
                );

                    $element->add_control(
                        'frame_color',
                        [
                            'label'     => esc_html__('Frame Color', 'cosmecos_plugin'),
                            'type'      => \Elementor\Controls_Manager::COLOR,
                            'selectors' => [
                                '{{WRAPPER}} .swiper-slide a:before' => 'border-color: {{VALUE}};'
                            ],
                            'condition' => [
                                'link_to'    => ['file', 'custom'],
                                'view_style' => 'style_2'
                            ]
                        ]
                    );

                $element->end_controls_tab();

                // ------------------------ //
                // ------ Active Tab ------ //
                // ------------------------ //
                $element->start_controls_tab(
                    'frame_hover',
                    [
                        'label'     => esc_html__('Hover', 'cosmecos_plugin'),
                        'condition' => [
                            'link_to'    => ['file', 'custom'],
                            'view_style' => 'style_2'
                        ]
                    ]
                );

                    $element->add_control(
                        'frame_color_hover',
                        [
                            'label'     => esc_html__('Frame Color on Hover', 'cosmecos_plugin'),
                            'type'      => \Elementor\Controls_Manager::COLOR,
                            'selectors' => [
                                '{{WRAPPER}} .swiper-slide a:hover:before' => 'border-color: {{VALUE}};'
                            ],
                            'condition' => [
                                'link_to'    => ['file', 'custom'],
                                'view_style' => 'style_2'
                            ]
                        ]
                    );

                $element->end_controls_tab();

            $element->end_controls_tabs();
        }
    }, 10, 3);
}